import logging
from easilyb.json_min_db import JsonMinConnexion
from threading import Lock

logger = logging.getLogger(__name__)
NOTHING = object()


def get_config_path(filename='config.yaml', appname=None, appauthor=None, appversion=None,
                    dirpath=None, app_root_config=None, user_config=None, system_config=None,
                    approot_config_dirname='config'):
    from os.path import join, dirname

    # There are 4 options for dirpath:
    #   0- given in dirpath argument
    #   1- application root dir
    #   2- user config dir
    #   3- system config dir
    if dirpath is None:
        if app_root_config is False and user_config is False and system_config is False:
            raise ValueError("Required one of: app_root_config, user_config, and system_config be True or None")
        if app_root_config is True or not (user_config or system_config):
            import sys
            dirpath = join(dirname(sys.modules['__main__'].__file__), approot_config_dirname)

        elif user_config is True or not system_config:
            import appdirs
            dirpath = appdirs.user_config_dir(appname, appauthor, appversion)
        else:
            import appdirs
            dirpath = appdirs.site_config_dir(appname, appauthor, appversion)

    return join(dirpath, filename)


class EasyConfig:
    def __init__(self, appname=None, appauthor=None, appversion=None,
                 filename=None, dirpath=None, app_root_config=None, user_config=None,
                 system_config=None,approot_config_dirname='config',
                 filepath=None, create=True, template=None, template_file=None, indent=3, driver="yaml"):
        if filepath is None:
            if filename is None:
                raise ValueError('Argument required: filepath or filename')
            filepath = get_config_path(filename, appname, appauthor, appversion, dirpath,
                                       app_root_config, user_config, system_config,
                                       approot_config_dirname)

        self.db = JsonMinConnexion(filepath, create=create, template=template, template_file=template_file,
                                   indent=indent, driver=driver)
        self._lock = Lock()
        self._save = True

    def get(self, key, default=NOTHING, set_default=True, save=True):
        try:
            return self.db[key]
        except:
            if default == NOTHING:
                raise KeyError(key)
            else:
                if set_default:
                    self.set(key, default, save=save)
                return default

    def set(self, key, value, save=True):
        if self._save:
            with self._lock:
                self.db[key] = value
                if save:
                    self.db.save()
        else:
            self.db[key] = value

    def save(self):
        self.db.save()

    def __enter__(self):
        self._lock.acquire()
        self._save = False
        self.db.__enter__()

    def __exit__(self, exc_type, exc_val, exc_tb):
        try:
            self.db.__exit__(exc_type, exc_val, exc_tb)
        finally:
            self._save = True
            self._lock.release()



#!/usr/bin/env python3
from distutils.core import setup
import os

VERSION = '0.19.0'  # major.minor.fix
                    # MAJOR incompatible API changes, MINOR add backwards-compatible functionality, FIX bug fixes


def package_files(directory):
    paths = []
    for (path, directories, filenames) in os.walk(directory):
        for filename in filenames:
            if not path.endswith('__pycache__') and not filename.endswith(".pyc"):
                paths.append(os.path.relpath(os.path.join(path, filename), directory))
    return paths


if __name__ == "__main__":
    extra_files = package_files('easilyb/')

    # print extra_files


    setup(
        name='easilyb',
        version=VERSION,
        description='Frequently used functions library for Python3 By Khalid Grandi (github.com/xaled).',
        long_description='Frequently used functions library for Python3 By Khalid Grandi (github.com/xaled).',
        long_description_content_type='text/x-rst',
        keywords='library utils common',
        author='Khalid Grandi',
        author_email='kh.grandi@gmail.com',
        url='https://github.com/xaled/easilyb/',
        install_requires=['requests', 'pycrypto', 'pyaml', 'lxml'],
        python_requires='>=3',
        packages=['easilyb'],
        package_data={'': extra_files},
    )



import sys
COLORS = {
    'blue': '\033[94m',
    'green': '\033[92m',
    'yellow': '\033[93m',
    'red': '\033[91m',
}
END_COLOR = '\033[0m'


def cprint(*args, sep=' ', end='\n', file=None, color=None):
    file = file or sys.stdout
    if color is not None and color in COLORS:
        args = list(args)
        args.insert(0, COLORS[color])
        args.append(END_COLOR)
    print(*args, file=file, sep=sep, end=end)


def prompt(message, match=None, match_function=None, choices=None, tries=10, default=None, allow_empty=False,
           color=None, trailing='? ', case_sensitive=False, description=None):
    t = 0
    prompt_message = message
    default_showed = default is None

    # choices
    if not case_sensitive and choices is not None:
        choices = [c.lower() for c in choices]
    if choices is not None:
        if default is not None and default in choices and not case_sensitive:
            prompt_message += ' [%s]' % '/'.join([c if c != default else c.upper() for c in choices])
            default_showed = True
        else:
            prompt_message += ' [%s]' % '/'.join(choices)
    # description & default value
    if description is not None:
        prompt_message += ' (%s)' % description
    elif not default_showed:
        prompt_message += ' (Default: %s)' % default

    # trailing
    prompt_message += trailing
    if not case_sensitive and choices is not None:
        choices = [c.lower() for c in choices]
    if color is not None and color in COLORS:
        prompt_message = COLORS[color] + prompt_message + END_COLOR
    while t < tries:
        data = input(prompt_message)
        if len(data.strip()) == 0 and default is not None:
            return default
        if choices is not None:
            if data in choices:
                return data
        elif match is not None:
            if match.match(data):
                return data
        elif match_function is not None:
            if match_function(data):
                return data
        else:
            if allow_empty or len(data.strip()) > 0:
                return data
        t += 1
    raise ValueError("Max tries reached! data entered by the user is not valid!")


